#File: ReadSynmap.r
#Written By: Daniel M. Matross
#Date: 3/7/2005
#-----------------------------
#File to read and determine fractions for SYNMAP land classes. It is assumed
#that there are 48 land classes, per Martin Jung's Email. These are stored in 
#a binary file on the grid. Goal is to read those classification schemes, assumed
#to be at 30-arc seconds.
#
#Scheme is to extract the appropriate subgrid 'on the fly' in order to save
#memory. The main file covers the entire globe with 17500 rows and 43200 columns
#meaning all latitudes and nearly all longitudes are covered at 30 Arcsec resolution,
#Plan is to read through at 500 rows per interval (35 total intervals), and 
#extract all those points that fall into specified subgrid, thus creating a 
#a 30 arcsecond database for the specified grid area. From there, we aggregate
#to the appropriate resolution.
#

#Function: ReadSynmap
#usage: subset30Arcsec<-ReadSynmap()
#-----------------------------------
#Reads binary file in pieces appropriate to available memory and extracts
#only the appropriate subgrid at original resolution. 
#
#Inputs--infile: binary data file
#        subLon: Longitude limits {west,east} of subgrid to extract, deg E
#        subLat: Latitude limits {north, south} of subgrid to extract, deg N
#
#Returns--object with 30 Arc second resolution, matrix of SYNMAP class values
#

ReadSynmap<-function(infile="/deas/group/stilt/Vegetation/synmap/synmap_LC_jan1",
			subLon=c(-145,-51),subLat=c(65,11))
{
	#Parameters for input file

	deg2syn<-120      #Assume 30 Arcsecond, i.e. 120 per degree
	synLeft<-(-180)   #Assume left at -180E
	synTop<-90        #Assume top at 90N
	ncolSyn<-43200    #Assume 43200 rows--360 deg longitude at 30 arcsecs.

	subLimLR<-round((abs(synLeft-subLon)*deg2syn)+c(1,0))
	subLimTB<-round((abs(synTop-subLat)*deg2syn)+c(1,0))
		
	sIndx<-subLimTB[1]:subLimTB[2]
	cIndx<-subLimLR[1]:subLimLR[2]

	#initialize

	subMat<-matrix(0,nrow=(subLimTB[2]-subLimTB[1])+1,
			 ncol=(subLimLR[2]-subLimLR[1])+1)
	#label top left corner
	dimnames(subMat)<-list(seq(subLat[1],subLat[2]+1/deg2syn,by=-1/deg2syn),
			       seq(subLon[1],subLon[2]-1/deg2syn,by=1/deg2syn))

	
	
	con<-file(infile,open="rb")
	for(j in 1:35) { #Assume 17,500 rows
		print(j)
		jIndx<-((j-1)*500+1):(j*500)
		
		if(min(jIndx)>subLimTB[2]) 
			next
		
		
		readMat<-matrix(nrow=500,ncol=ncolSyn)
		for(i in 1:500) {
			readDat<-readBin(con, "int", n=ncolSyn,size=1,signed=TRUE)
			readMat[i,]<-readDat
		}
		
		#determine matches
		aMatch<-match(jIndx,sIndx)
		if(length(which(!is.na(aMatch)))==0) 
			next
		
		sMatch<-aMatch[which(!is.na(aMatch))]
		jMatch<-which(is.element(jIndx,sIndx))
		if(length(which(!is.na(jMatch)))==0) 
			next		
		
		#store
		subMat[sMatch,]<-readMat[jMatch,cIndx]	

		rm(readMat)
		#print(paste(round(100*j/35,2), "% Complete"))	
	}
	close(con)
	return(subMat)
} 

#Function: SortandFracSynmap
#usage:
#----------------------------
#Function to take 30arcsecondx30arcsecond grid, aggregate and determine fraction
#of each vegetation type in the synmap scheme.
#

SortandFracSynmap<-function(inMat,ll.x=-145,ll.y=11,numpix.x=376,numpix.y=324,xres=1/4,yres=1/6,nclss=48,
				reclssH2O=FALSE)
{
test<-0
	if(reclssH2O) {
		matcher<-1:nclss
	} else {
		matcher<-0:(nclss-1)
	}

	#Subset vector that corresponds to gridbox
	bigarray<-array(0,c(numpix.y,numpix.x,nclss))
	
	xx<-1:numpix.x
	yy<-1:numpix.y
	l.xx<-seq(ll.x,by=xres,length=numpix.x)
	l.yy<-seq(ll.y,by=yres,length=numpix.y)

	nbox.x<-120*xres #assume inMat has 30 arcsecond res
	nbox.y<-120*yres 
	
	inLon<-as.numeric(dimnames(inMat)[[2]])
	inLat<-as.numeric(dimnames(inMat)[[1]])
	inLat<-inLat-0.0001 #Rounding error on conversion

	for(ii in 1:numpix.x) {
#	for(ii in 323:numpix.x) {
		for(jj in 1:numpix.y) {
		subMat<-inMat[which(inLat>l.yy[jj] & inLat<=l.yy[jj]+yres),
					which(inLon>=l.xx[ii] & inLon<l.xx[ii]+xres)]
			if(nrow(subMat)==0|ncol(subMat)==0) {
				print("DMM SKIP")
				next
			}
			
			#if(ncol(subMat)!=nbox.x|nrow(subMat)!=nbox.y) {
                                #print(paste("X",nbox.x,ncol(subMat),sep=":"))
                               # print(paste("Y",nbox.y,nrow(subMat),sep=":"))
				#warning("Dimensions off on subsetting")
				#next
			#}
			
			#if(reclssH2O) inMat[which(inMat==0)]==nclss
			
			subVec<-as.vector(subMat)
	#Determine fraction
			sVec<-sort(subVec)
			indxNums<-match(matcher,sVec,nomatch=999) #gives postition of first index in sorted vector of that class, will 'diff' to get number of each class
			indxNums<-c(indxNums,length(sVec)+1) #account for last veg class, a diff will now yield vector with proper number of classes and proper accounting on last class	
			indxNums<-rev(cummin(rev(indxNums))) #account for any non-matched classes by taking index of NEXT clss and applying it--will give 0 in diff	
			nCnt<-diff(indxNums) #number in each class
			fCnt<-nCnt/length(sVec) #fraction of each veg class
#			bigarray[ii,jj,]<-fCnt
			bigarray[jj,ii,]<-fCnt
		}
		print(paste(round(100*ii/numpix.x,2),"% complete"))
                test<-round(100*ii/numpix.x,2)
	}
	return(bigarray)
}

#_______________________________________________________________________________________________
#ReadSynmap.r
#Written By: Daniel M. Matross
#Date: 3/8/2005

#Notes from chg:
#
#Date: Thu, 24 Feb 2005 17:25:21 +0100
#From: Christoph Gerbig <cgerbig@bgc-jena.mpg.de>
#To: Daniel M. Matross <matross@fas.harvard.edu>
#Cc: Steven Wofsy <wofsy@fas.harvard.edu>, Victoria Y. Chow <vchow@fas.harvard.edu>,
#     John Lin <jcl@atmos.colostate.edu>, Mahadevan Pathmathevan <devan@deas.harvard.edu>
#Subject: Re: GLC2000 Default Page
#
#Dan et al.,
#
#I put the synmap binary file into the directory 
#/deas/group/stilt/Vegetation/synmap/ together with a header file that 
#contains some info.
#
#According to Martin, the map is in simple flat binary (43200 cols and 
#17500 rows); projection: geographic, datum wgs84; grid cell size is 
#0.008333; upper left corner is -180 East, 90 North. It opens in ENVI 
#(don't know whether you have that).
#
#You can read the first 1000 lines it with a binary read in R:
#sm<- file("/deas/group/stilt/Vegetation/synmap/synmap_LC_jan1", "rb")
#dat<-NULL
#for(i in 1:1000){
#  #it is a binary file, with 1 byte integers
#  tmp<-readBin(sm, "int", 43200,size=1) #byte 43200*17500
#  dat<-rbind(dat,tmp)
#}
#close(sm)

#you need to read in in pieces, since it is a bit large. I leave the 
#gridding up to you, let me know if you need help.
