#
# R script for preprocessing EVI/LSWI indices for the
# Vegetation Photosynthesis and Respiration Model (VPRM)
# obtained from NASA MODIS product (MOD09A1).
#
#
#
# This version of the script is supposed to be used with the WRF-VPRM model.
# In order to work it needs WRF input NETCDF files containing the target
# grid definition. (wrfinput_d01, wrfinput_d02, etc.)
#
#
#
# written by
# Christoph Gerbig (CHG), MPI BGC Jena, Germany (cgerbig@bgc-jena.mpg.de)
# Roberto Kretschmer (ROK), MPI BGC Jena, Germany (rkretsch@bgc-jena.mpg.de)
#
#
#
# Descriptions:
#
# Mahadevan, P., S. Wofsy, D. M. Matross, X. Xiao, A. Dunn, J. C. Lin, C.
# Gerbig, J. W. Munger, V. Y. Chow, and E. Gottlieb, A Satellite-Based
# Biosphere Parameterization for Net Ecosystem CO2 Exchange: Vegetation
# Photosynthesis and Respiration Model (VPRM), Global Biogeochem. Cycles,
# 22, GB2005, doi:10.1029/2006GB002735, 2008.
#
# Ahmadov R; Gerbig C; Kretschmer R; Koerner S; Neininger B; Dolman AJ;
# Sarrat C (2007): Mesoscale covariance of transport and CO2 fluxes:
# evidence from observations and simulations using the WRF-VPRM coupled
# atmosphere-biosphere model. Journal of Geophysical Research -
# Atmospheres 112, D22107
#
# Ahmadov, R., Gerbig, C., Kretschmer, R., K�rner, S., R�denbeck, C.,
# Bousquet, P., and Ramonet, M.: Can we use hourly CO2 concentration data
# in inversions? Comparing high resolution WRF-VPRM simulations with
# coastal tower measurements of CO2, Biogeosciences Discuss., 5,
# 4745-4776, 2008.
#
#


scriptpath <- getwd() # path of preprocessor
sourcepath <-paste(scriptpath,"/RSources/",sep="")

source(paste(scriptpath,"/RSources/monthdayyear.r",sep=""))

# read configuration
source(paste(scriptpath,"/config.r",sep=""))

ftdata   <- TRUE          # shall we connect to ftp and check for data?
projtype <- "LCC"          # type of projection LCC xor GEO
datp <- rdataout
mdy      <- month.day.year
attachn  <- function(what, pos = 2){if(!(paste("file:",what,sep="")%in%search()))attach(what,pos)}
go       <- graphics.off

# set synmap resolution
spacing <- 1/120

# set number of used vegetation classes
nveg <- 8 

# create working directories
if (!file.exists(mpath)) dir.create(mpath,recursive=TRUE)
if (!file.exists(rdatain)) dir.create(rdatain,recursive=TRUE)
if (!file.exists(rdataout)) dir.create(rdataout,recursive=TRUE)

# check for synmap
if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
  #stop(paste("Synmap ",rdatain,"/.RDatasynmap.",synmaptouse," not found. Use ",sourcepath,"/gridEurope.r to obtain synmap!",sep=""));
  print("SYNMAP not there. Prepare one...")
  source(paste(scriptpath,"/RSources/getSynmap.r",sep=""))
  if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
   stop("Could not prepare SYNMAP!")
  }
}

setwd(rdataout)

#library("hdf5")
library("h5r")

source(paste(scriptpath,"/RSources/assignr.r",sep=""))
source(paste(scriptpath,"/RSources/project.nice.r",sep=""))
source(paste(scriptpath,"/RSources/getminmax.r",sep=""))
source(paste(scriptpath,"/RSources/distance.r",sep=""))
source(paste(scriptpath,"/RSources/getr.r",sep=""))
source(paste(scriptpath,"/RSources/loess.r",sep=""))
source(paste(scriptpath,"/RSources/imagell.r",sep=""))
source(paste(scriptpath,"/RSources/na_replace.r",sep=""))
source(paste(scriptpath,"/RSources/combineTiles.r",sep=""))
source(paste(scriptpath,"/RSources/hdfread.r",sep=""))
source(paste(scriptpath,"/RSources/parseNamelist.r",sep=""))
source(paste(scriptpath,"/RSources/getModis.r",sep=""))
source(paste(scriptpath,"/RSources/getModisNRT.r",sep=""))
source(paste(scriptpath,"/RSources/checkModisRepos.r",sep=""))
source(paste(scriptpath,"/RSources/checkModisReposNRT.r",sep=""))
source(paste(scriptpath,"/RSources/extractAndReduce.r",sep=""))
source(paste(scriptpath,"/RSources/doResampling.r",sep=""))
source(paste(scriptpath,"/RSources/readIndicesIntoR.r",sep=""))
source(paste(scriptpath,"/RSources/cutToDomain.r",sep=""))
source(paste(scriptpath,"/RSources/doAggregation.r",sep=""))
source(paste(scriptpath,"/RSources/writeOutput.r",sep=""))




# get additional configuration settings
# configuration is saved in a R-object file called ".RDataprojinfo"
if(!file.exists(paste(datp,".RDataprojinfo",sep="")))parseNamelist()

# load grid configuration
projinfo <- getr("projinfo",datp)
fcstmode<-FALSE
if(exists("forecstTF"))if(forecstTF)fcstmode<-TRUE
testmode<-FALSE
if(exists("testTF"))if(testTF)testmode<-TRUE
if(fcstmode){projinfo$year<-year;assignr("projinfo",projinfo,datp);projinfo <- getr("projinfo",datp)}#use year specified in config.r

# set 8 day intervals to process
# to do: calculate from start and end date set in namelist!
#doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"009",sep="")),as.numeric(paste(projinfo$year,"361",sep="")),by=8)
#doyuse <- seq(as.numeric(paste(projinfo$year,"041",sep="")),as.numeric(paste(projinfo$year,"305",sep="")),by=8)

#new 05.07.2010 (tk)
doyuse <- doyall  <- seq(as.numeric(paste(projinfo$year,"001",sep="")),as.numeric(paste(projinfo$year,projinfo$ldoy,sep="")),by=8)

doyuse<-doyuse[doyuse!=2001169] #exclude missing MODIS acquisition period

# cd to working directory
setwd(mpath)

# - check for MODIS data
# - make sure we have all the tiles needed for the configuration
#
# A tile lookup table (TLUT) is created and returned by the function below.
# The TLUT contains all tiles needed for the domain
# and later on also the tile coordinates in lat/lon, calculated during resampling.
# But for now TLUT only contains MODIS tile IDs (e.g. h17v04).

if (!file.exists("TLUT.RData")) TLUT <- getModis(doyuse)
if (file.exists("TLUT.RData")) load("TLUT.RData")

################################################################################################################################################################
if(testmode){
#only for testing, to avoid daily updating of MODIS data
  newModis<-TRUE
  skip<-FALSE
  #assignr("doyuse",doyuse,mpath)
  doyuse<-getr("doyuse",mpath)
}
if(!testmode){
  #get additional timesteps
  if(fcstmode){
    #check which are available from standard MODIS and which are needed from NRT MODIS
    checkModis(TLUT,year,doyuse) #update standard MODIS download (TLUT was still there, so getModis was not called before
    lastModis<-checkModisRepos(TLUT,year,doyuse)
    #get current date as day of year (counting jan 1 as 1
    jd<-julian(as.numeric(format(Sys.time(), "%m")),as.numeric(format(Sys.time(), "%d")),as.numeric(format(Sys.time(), "%Y")),origin=c(1,0,year))
    doyuseNRT<-projinfo$year*1000+c(seq(lastModis+8,jd-2,8),jd-1) #use data from yesterday, those are the latest in NRT
    doyuseNRT<-checkModisNRT(TLUT,year,doyuseNRT) #download MODIS NRT data; if last day not available, day closest to that and update doyuseNRT

    nrtModis<-checkModisReposNRT(TLUT,year,doyuseNRT)
    if(any(is.na(nrtModis))){ #this could happen if the day's directory is there, but the files have not yet been uploaded completely
      nrtModis.tilemiss<-apply(nrtModis,1,function(x)any(is.na(x)))#check which tile is not available all days
      nrtModis.daymiss<-apply(nrtModis,2,function(x)any(is.na(x)))#check which day is incomplete
      daymiss<-unique(nrtModis[!nrtModis.tilemiss,nrtModis.daymiss])
      print(paste("MODIS NRT day",daymiss,"still missing (probably not yet uploaded completely), getting day before"))
      doyuseNRT[doyuseNRT-projinfo$year*1000==daymiss]<-doyuseNRT[doyuseNRT-projinfo$year*1000==daymiss]-1
      doyuseNRT<-checkModisNRT(TLUT,year,doyuseNRT) #download MODIS NRT data; if last day not available, day closest to that and update doyuseNRT
    }
    doyuse <- c(doyuse[doyuse<=projinfo$year*1000+lastModis],doyuseNRT)

    #clean up Modis repository to only keep currently required NRT files, otherwise they accumulate
    fls.delete<-list.files(paste(modisrep,year,sep=""),pattern="MOD09A1N.")
    for(pat in c((projinfo$year*1000+lastModis):(doyuseNRT[length(doyuseNRT)]+1)))fls.delete<-fls.delete[!grepl(pat,fls.delete)]#also don't delete recent files
    #if(length(fls.delete)>0)file.remove(paste(modisrep,year,"/",fls.delete,sep=""))
  }
  save(doyuse,file="doyuse.RData") #save for use in loess filtering

  newModis<-FALSE
  if (file.exists("TLUT.RData")&exists("prepr.additionalTF")){#also need to get MODIS data from rest of year, if only part of year was processed before
    if(prepr.additionalTF&!exists("forecstTF")){checkModis(TLUT,year,doyuse);newModis<-TRUE}
    if(exists("forecstTF"))if(prepr.additionalTF&!forecstTF){checkModis(TLUT,year,doyuse);newModis<-TRUE}
    if(exists("forecstTF"))if(prepr.additionalTF&forecstTF){newModis<-TRUE}#for forecast mode the download has been done in the previous code section
  }


  # check for synmap
  if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
    stop(paste("Synmap ",rdatain,"/.RDatasynmap.",synmaptouse," not found. Use ",sourcepath,"/gridEurope.r to obtain synmap!",sep=""));
  }

  # check if we have to initialize MODIS fields (read into R)
  skip <- TRUE

  for (ind in indices) {
   if (length(list.files(path=rdataout,pattern=glob2rx(paste(".*",ind,"*combined",sep="")),all.files=T))!=length(doyuse)) skip<-FALSE
   if(fcstmode)skip<-FALSE #will always have new NRT MODIS data ...
  }


  if (!skip) {
   # delete old resample log
   if (file.exists("resample.log")) system("rm resample.log")

   # Extracting EVI and LSWI and reducing MODIS data to 1km.
   for (ind in indices) extractAndReduce(ind,TLUT,projinfo$year,doyuse) # defined in extractAndReduce.r
   # clean up NRT files that are not needed
   if(fcstmode){
    fls.delete<-list.files("./",pattern="_1km_MOD09A1N.")
    for(pat in c(doyuseNRT[1]:(doyuseNRT[length(doyuseNRT)]+1)))fls.delete<-fls.delete[!grepl(pat,fls.delete)]#also don't delete recent files
    #if(length(fls.delete)>0)file.remove(paste("./",fls.delete,sep=""))
   }


   # Resample EVI/LSWI from SIN to GEO (LAT/LON) projection.
   # LL and UR corners are added to TLUT for every tile (needed only once).
   # Adding happens only if resampling is done for the first time. If the resampled files are
   # already there, this step is skipped, so TLUT has to be loaded (see below).

   for (ind in indices) TLUT <- doResampling(ind,TLUT,projinfo) # defined in doResampling.r

   # load old TLUT if existing, else save current one.
  #####!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  #  this need fixing; when previous run crashed, TLUT[,3:6] are all zero!
  #####!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
   #if (file.exists("TLUT.RData")) load("TLUT.RData") else save(TLUT,file="TLUT.RData")
   save(TLUT,file="TLUT.RData")
  #####!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  #####!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

   print(TLUT)
   # convert EVI/LSWI HDF files to R objects.
   for (ind in indices) readIndicesIntoR(ind,projinfo,doyuse) # defined in readIndicesIntoR.r
   gc()
  }

}#of if(testmode){
################################################################################################################################################################

print("Now smooting")
###################### SMOOTHING: loess filtering ##############################
# to do: which times to exclude because of snow? (use special MODIS data for SNOW)
# to do: optimize performance

if (!skip) {
 for (ind in indices) {
  # check if already done
  if (length(list.files(path=mpath,pattern=glob2rx(paste(ind,"*smooth*.nc",sep=""))))==length(doyuse)) next
 
  # do it in parallel?
  if (do_parallel) {
   system(paste(scriptpath,"/submit_loess.sh ",scriptpath," '",ind,"'",sep=""))
  } else {
   # do it per tile
   for (i in 1:(length(TLUT)/6)) {
    if ((length(TLUT)/6)==1) {
     if (TLUT[1]<10) H <- paste("0",TLUT[1],sep="") else H <- TLUT[1]
     if (TLUT[2]<10) V <- paste("0",TLUT[2],sep="") else V <- TLUT[2]
    } else {
     if (TLUT[i,1]<10) H <- paste("0",TLUT[i,1],sep="") else H <- TLUT[i,1]
     if (TLUT[i,2]<10) V <- paste("0",TLUT[i,2],sep="") else V <- TLUT[i,2]
    }
    projs<-paste("h",H,"v",V,".",projinfo$year,".ier1km",sep="")

    Smoothevilswi(ind,proj=projs,doyall=doyall,doyuse=doyuse,fcstmode=fcstmode) # defined in loess.r
   }
   rm(projs);gc()
  }
 }
}


print("Now combining titles")
############################# combining tiles ##################################
if (!skip) {
 for (ind in indices) combineTiles(ind,TLUT,doyuse,projinfo,fcstmode=fcstmode)  # defined in combineTiles.r
 gc()
}

######## extract evi/lswi indices and vegetation map for target grid ###########
######## gap filling
print("Now extract evi/lswi indices")
for (dom in 1:length(projinfo$dxy)) {
 for (ind in indices) cutToDomain(ind,dom,doyuse,projinfo,fcstmode=fcstmode) # defined in cutToDomain.r
}
gc()

################# aggregation and projection to target grid ####################
print("Now aggregation and projection")
for (dom in 1:length(projinfo$dxy)) {
 for (ind in indices) doAggregateLCC(ind,dom,doyuse,nveg,fcstmode=fcstmode)   # defined in doAggregation.r
}
gc()

########## calculate min and max indices #######################################
print("Now calculate min and max indices")
for (dom in 1:length(projinfo$dxy)) {
 for (ind in indices) getMinMaxIndices(ind,projinfo$year,dom,doyuse,fcstmode=fcstmode) # defined in getminmax.r
}
gc()


########### write NETCDF files #################################################
print("Now write netCDF files")
# generate output directory 
start_day_of_year<-min(doyuse-projinfo$year*1000)

resdir <- paste(outdir,"/","VPRM_input_",domainname,"_",projinfo$year,sep="")
if (!file.exists(resdir)) dir.create(resdir)

for (dom in 1:length(projinfo$dxy)) {
 writeVegetationMap(dom,projinfo,nveg,resdir)
 for (ind in indices) writeIndices(ind,dom,projinfo,start_day_of_year,doyuse,nveg,resdir)
}
gc()


######### All done. ############################################################
print("Preprocessing of VPRM indices finished.")
print(date())







# used VPRM classes:
# 1 Trees evergreen
# 2 Trees deciduous
# 3 Trees mixed
# 4 Trees and shrubs
# 5 Trees and grasses
# 6 Trees and crops
# 7 grasses
# 8 Barren, Urban and built-up, Permanent snow and ice
# c("trees evergreen", "trees deciduous", "trees mixed", "trees and shrubs", "trees and grasses", "trees and crops", "grasses", "barren, urban and built-up, permanent snow and ice")

#SYNMAP classes:
#Trees Needle Evergreen Trees
#Trees Needle Deciduous
#Trees Needle Mixed
#Trees Broad Evergreen
#Trees Broad Deciduous
#Trees Broad Mixed
#Trees Mixed Evergreen
#Trees Mixed Deciduous
#Trees Mixed Mixed
#Trees and shrubs Needle Evergreen
#Trees and shrubs Needle Deciduous
#Trees and shrubs Needle Mixed
#Trees and shrubs Broad Evergreen
#Trees and shrubs Broad Deciduous
#Trees and shrubs Broad Mixed
#Trees and shrubs Mixed Evergreen
#Trees and shrubs Mixed Deciduous
#Trees and shrubs Mixed Mixed
#Trees and grasses Needle Evergreen
#Trees and grasses Needle Deciduous
#Trees and grasses Needle Mixed
#Trees and grasses Broad Evergreen
#Trees and grasses Broad Deciduous
#Trees and grasses Broad Mixed
#Trees and grasses Mixed Evergreen
#Trees and grasses Mixed Deciduous
#Trees and grasses Mixed Mixed
#Trees and crops Needle Evergreen Crops/natural vegetation mosaic
#Trees and crops Needle Deciduous
#Trees and crops Needle Mixed
#Trees and crops Broad Evergreen
#Trees and crops Broad Deciduous
#Trees and crops Broad Mixed
#Trees and crops Mixed Evergreen
#Trees and crops Mixed Deciduous
#Trees and crops Mixed Mixed
#Shrubs and crops
#Grasses and crops
#Crops
#Shrubs
#Shrubs and grasses
#Shrubs and barren
#Grasses
#Grasses and barren
#Barren
#Urban and built-up
#Permanent snow and ice

# MAPPING
#VPRM	Synmap
#1	1
#2	2
#3	3
#1	4
#2	5
#3	6
#1	7
#2	8
#3	9
#4	10
#4	11
#4	12
#4	13
#4	14
#4	15
#4	16
#4	17
#4	18
#5	19
#5	20
#5	21
#5	22
#5	23
#5	24
#5	25
#5	26
#5	27
#6	28
#6	29
#6	30
#6	31
#6	32
#6	33
#6	34
#6	35
#6	36
#4	37
#4	38
#4	39
#4	40
#7	41
#7	42
#7	43
#6	44
#8	45
#8	46
#8	47
#8	48
